#ifndef _1WIRE_																//    
#define _1WIRE_

#include <avr/iom128.h>														//  /

#ifndef F_CPU
# warning "F_CPU not defined. Frequency by default 14.7456MHz"
#define F_CPU 14745600UL													/*     (     )*/
#endif

#define DDR  DDRE
#define PORT PORTE
#define PIN  PINE
#define PB   PE4

#define	_W1_PORT_S	{DDR|=(1<<PB); PORT|=(1<<PB);}							/*     */
#define _W1_PORT_R	{DDR|=(1<<PB); PORT&=~(1<<PB);}							/*     */

#define _W1_PORT_IN	{DDR&=~(1<<PB); PORT|=(1<<PB);}							/*      */
#define _W1_PIN		(PIN & (1<<PB))>>PB										/*   */

#include <util/delay.h>														//   

//==========================    1-Wire =============================
// :
//1-      , 0-    .

char w1_init(void)
{
	char presence;															//      
	
	_W1_PORT_R;																//  
	_delay_us(480);															//  480  ( datasheet)
	_W1_PORT_IN;															//   
	_delay_us(70);															//  70  (    )
	presence=_W1_PIN  ^ 0x01;												//     
	_delay_us(410);															//  410  (      960 )

	return presence;														// 
}

//===================       ( ) ====================
// :
//  

char _Read_Time_Slot(void)
{
	char w1_bit=0;															//   
	__asm__ __volatile__ ("cli");											//  
	_W1_PORT_R;																//     
	_delay_us(2);															//  1  (   )
	_W1_PORT_IN;															//   
	_delay_us(15);															// 15  (     )
	if (_W1_PIN) w1_bit=1;													//    ,     
	_delay_us(45);															// 45  ( )
	__asm__ __volatile__ ("sei");											//  
	_delay_us(2);															//  1  (  )
	return w1_bit;
}

//==================       ( ) ====================
// :
//      

void _Write_Time_Slot(char w1_bit)
{
	__asm__ __volatile__ ("cli");											//  
	_W1_PORT_R;																//     
	_delay_us(2);															//  1  (   )
	if (w1_bit) _W1_PORT_S													//  "1",     
	else _W1_PORT_R;														//   
	_delay_us(60);															// 60  (  )
	_W1_PORT_IN;															//   
	__asm__ __volatile__ ("sei");											//  
	_delay_us(2);															//  1  (   )
}

//==========================      1Wire =============================
// :
//  

char w1_read(void)
{
	char result=0;															//   
	int temp;																// 
	
	for (temp=0; temp<8; temp++)											//  8  
	{
		result>>=1;															//    
		result|=_Read_Time_Slot()<<7;										//  
	}
	
	return result;															// 
}

//==========================      1Wire ============================
// :
//data-     

void w1_write(char data)
{
	int temp;																// 
	
	for (temp=0; temp<8; temp++)											//  8  
	{
		_Write_Time_Slot(data & 0x01);
		data>>=1;															//   
	}
}

//========================      1Wire ==========================
/*	       1Wire    Reset, Search ROM.    ,
  64         .      3- .        
   , ,   ,       .  ,    
 ,   2- .    01,           0.     10, , ,
   1   .    2  11,   ,     .  ,       00,
  ,     , ..     ,       1,   ,  0   .
	 ,     ,        : 0  1.  ,   
   ,        .
	         .        
 Reset, Search ROM.             ,   ,   
    .           . ,  
      ,      .

	       1Wire   .           . , 
   ,   .         , 
       .         :

-         .      ,       
  .        ,      ,     
         .         ,    ,   
 ,       .            ;

-             ,      ,
  .     ,     ,       
, ..       .     ,      ,  
      .  ,           , 
    ;

-             ( ,        ),   
    -    .*/

// :
//*p-        
// :
//    

int w1_search(void *p)
{
	__volatile__ int count_bit;												//  
	__volatile__ unsigned long Control_Number[2]={0,0};						//  (64 )
	__volatile__ int Control_Length=64;										//   (    )
	__volatile__ int Count_Collision;										// 
	__volatile__ char Status;												//        ( )
	__volatile__ int Count_Device=0;										//  
	__volatile__ int temp;													// 
	__volatile__ char *pp;													// 
	
	pp=p;																	//    
	
	char address_device[8];													//  
	
	while (Control_Length!=0)												//     ,    
	{
		Count_Collision=0;													//  

		if (w1_init()==0) return 0;								            //       ,      
		w1_write(0xF0);												        // " "				
	
		for (count_bit=0; count_bit<64; count_bit++)						//  64    
		{
			Status=_Read_Time_Slot()<<1;									//  
			Status|=_Read_Time_Slot();										//  				
				
			switch (Status)													//    
			{
			case 0:															//   (         )
				Count_Collision++;											//  
					
				//       ,     
				if (Count_Collision>Control_Length) 
				{
					Control_Length++;										//   
					Control_Number[1]<<=1;									// 64- 
					if (Control_Number[0] & 0x80000000) Control_Number[1]|=0x00000001;
					Control_Number[0]<<=1;									//    
				}

				//     
				if (Count_Collision>31) temp=(Control_Number[1]>>(Control_Length-Count_Collision)) & 0x00000001;	
				else temp=(Control_Number[0]>>(Control_Length-Count_Collision)) & 0x00000001;	
					
				if (temp==0)												//   (       )
				{
					_Write_Time_Slot(0);									//   
					address_device[count_bit>>3]&=~(1<<(count_bit & 0x07));	//    						
				}
				else
				{
					_Write_Time_Slot(1);									//   
					address_device[count_bit>>3]|=1<<(count_bit & 0x07);	//    						
				}
				break;
			case 1:															//        0
				_Write_Time_Slot(0);										//   
				address_device[count_bit>>3]&=~(1<<(count_bit & 0x07));		//    
				break;					
			case 2:															//        1
				_Write_Time_Slot(1);										//   
				address_device[count_bit>>3]|=1<<(count_bit & 0x07);		//    
				break;					
			case 3:															//     
				return 0;
			}
		}
			
//-----------------------------    -----------------------------
		if (Count_Device==0) Control_Length=Count_Collision;				//    

		while ((Control_Number[0] & 0x00000001)==1)							//       
		{
			Control_Number[0]>>=1;											// 64         
			if (Control_Number[1] & 0x00000001) Control_Number[0]|=0x80000000;
			Control_Number[1]>>=1;
			Control_Length--;												//   
			if (Control_Length==0) break;									//     ,       
		}
		Control_Number[0]|=0x01;											//    ,    

//-----------------------------     -----------------------------
		for (temp=0; temp<8; temp++) 
		{
			*pp=address_device[temp];
			pp++;
		}
		Count_Device++;														//   
	}
	return Count_Device;													// 
}

//================        ===================
// :
//*p-        
//n-      
// :
//  

char w1_crc8(void *p, char n)
{
	char *pp;																// 
	char buf[8];															//  (        )
	char CRC=0;																//  
	char a=0, b=0;															//    
	int temp,temp2;															// 
	
	pp=p;																	//    
	for (temp=0; temp<n; temp++)											//        (      )
	{
		buf[temp]=*pp;
		pp++;
	}

//---------------------  CRC     X8+X5+X4+1 ----------------------
	for (temp=0; temp<n; temp++)
		for (temp2=0; temp2<8; temp2++)
		{
			a=buf[temp];
			a^=CRC;
			if ((a & 0x01)!=0) CRC=((CRC^0x18)>>1) | 0x80;
			else CRC>>=1;
			a=(a>>1) | b;
			if ((buf[temp] & 0x01)!=0) b=0x80;
			else b=0;
			buf[temp]>>=1;
		}
	
	return CRC;																//  CRC
}

//================       ===================
// :
//     10.

void Temperature_Board_Start(void)
{
	w1_init();																// 
	w1_write(0xCC);															// " "
	w1_write(0x44);															// " "
}

//================       ===================
// :
//   int,     
// :
//0-    , 1-    

uint8_t Temperature_Board_Read(int* Temperature)
{
	char _buf[9];															//    
	int temp;																// 	
	
	w1_init();																// 
	w1_write(0xCC);															// " "
	w1_write(0xBE);															// "  "

	for (temp=0; temp<9; temp++) _buf[temp]=w1_read();						//  

	if (_buf[8]==w1_crc8(_buf,8))											//  
	{
		*Temperature=(_buf[0]+(_buf[1]<<8))*0.625;							//    
		return 1;
	}
	else																	//    
	{
		*Temperature=0;
		return 0;
	}
}
#endif
